/*
 *
 * Definitions for mount interface. This describes the in the kernel build 
 * linkedlist with mounted filesystems.
 *
 * Author:  Marco van Wieringen <mvw@planets.elm.net>
 *
 * Version: $Id: mount.h,v 2.0 1996/11/17 16:48:14 mvw Exp mvw $
 *
 */
#ifndef _LINUX_MOUNT_H
#define _LINUX_MOUNT_H
#ifdef __KERNEL__

#include <linux/list.h>
#ifdef CONFIG_FUMOUNT
#include <linux/rwsem.h>
#endif

#define MNT_NOSUID	1
#define MNT_NODEV	2
#define MNT_NOEXEC	4
#define MNT_FUMOUNT 1<<16 /* set when forcibly removing mount */

struct vfsmount
{
	struct list_head mnt_hash;
	struct vfsmount *mnt_parent;	/* fs we are mounted on */
	struct dentry *mnt_mountpoint;	/* dentry of mountpoint */
	struct dentry *mnt_root;	/* root of the mounted tree */
	struct super_block *mnt_sb;	/* pointer to superblock */
	struct list_head mnt_mounts;	/* list of children, anchored here */
	struct list_head mnt_child;	/* and going through their mnt_child */
	atomic_t mnt_count;
	int mnt_flags;
	int mnt_expiry_mark;		/* true if marked for expiry */
	char *mnt_devname;		/* Name of device e.g. /dev/dsk/hda1 */
	struct list_head mnt_list;
	struct list_head mnt_fslink;	/* link in fs-specific expiry list */
	struct namespace *mnt_namespace; /* containing namespace */
#ifdef CONFIG_FUMOUNT
        struct rw_semaphore mnt_close_sem;
#endif
};

#ifdef CONFIG_FUMOUNT
extern void fs_fumount_mark_files(struct vfsmount *);

/* mntget checks that the parameter is not NULL, and now checks to
   see that the mount is not subject to a
   pending forced unmount.  If both checks pass, then the reference
   count for the mount structure is atomically incremented and the
   mount structure pointer is returned.  Otherwise, NULL is returned.
*/

static inline struct vfsmount *mntget(struct vfsmount *mnt)
{
        if ( (mnt) && !( mnt->mnt_flags & MNT_FUMOUNT ) ){
                atomic_inc(&mnt->mnt_count);
        }
        else {
                mnt = (struct vfsmount *) NULL;
        }
        return mnt;
}
#else
static inline struct vfsmount *mntget(struct vfsmount *mnt)
{
	if (mnt)
		atomic_inc(&mnt->mnt_count);
	return mnt;
}
#endif /*CONFIG_FUMOUNT*/

extern void __mntput(struct vfsmount *mnt);

static inline void _mntput(struct vfsmount *mnt)
{
	if (mnt) {
		if (atomic_dec_and_test(&mnt->mnt_count))
			__mntput(mnt);
	}
}

static inline void mntput(struct vfsmount *mnt)
{
	if (mnt) {
		mnt->mnt_expiry_mark = 0;
		_mntput(mnt);
	}
}

extern void free_vfsmnt(struct vfsmount *mnt);
extern struct vfsmount *alloc_vfsmnt(const char *name);
extern struct vfsmount *do_kern_mount(const char *fstype, int flags,
				      const char *name, void *data);

struct nameidata;

extern int do_add_mount(struct vfsmount *newmnt, struct nameidata *nd,
			int mnt_flags, struct list_head *fslist);

extern void mark_mounts_for_expiry(struct list_head *mounts);

extern spinlock_t vfsmount_lock;

#endif
#endif /* _LINUX_MOUNT_H */
